/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.listeners;

import static org.ow2.petals.bc.filetransfer.FileTransferConstants.DEFAULT_POLLING_PERIOD;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_FILETRANSFER_TRANSFERMODE_ATTACHMENTS;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_FILETRANSFER_TRANSFERMODE_SOURCE;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_POLLING;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_TRANSFERMODE;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_WASTE;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_WORKING;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.logging.Level;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * Listen to a directory
 * 
 * @author alouis
 * 
 */
public class FileTransferExternalListener extends AbstractExternalListener {

    private DirectoryPoller poller;

    private File workspace;

    private File working;

    private File waste;

    /**
     * Return the workspace directory
     * 
     * @return
     */
    public File getWorkspaceDirectory() {
        if (workspace == null) {
            workspace = new File(getComponent().getContext().getWorkspaceRoot());
        }
        return workspace;
    }

    /**
     * Return the working directory
     * 
     * @return
     * @throws PEtALSCFException
     *             no working directory specified in the SU
     */
    public File getWorkingDirectory() throws IOException {
        if (working == null) {
            // retrieve the directory to listen
            String directory = getExtensions().getValue(FILETRANSFER_WORKING);
            if (StringHelper.isNullOrEmpty(directory)) {
                String error = FILETRANSFER_WORKING + " parameter not set in the ServiceUnit.";
                getLogger().severe(error);
                throw new FileNotFoundException(error);
            }

            directory = FileTransferUtils.evaluateFilePath(directory);
            working = new File(directory);
            if (!working.exists()) {
                throw new FileNotFoundException(FILETRANSFER_WORKING + " parameter with value '"
                        + directory + "' is invalid");
            }
        }
        return working;
    }

    /**
     * Return the waste directory. Default one is the component workspace
     * directory
     * 
     * @return
     */
    public File getWasteDirectory() {
        if (waste == null) {
            // retrieve the directory to listen
            String s_directory = getExtensions().getValue(FILETRANSFER_WASTE);
            if (StringHelper.isNullOrEmpty(s_directory)) {
                s_directory = getComponent().getContext().getWorkspaceRoot();
            }
            waste = new File(s_directory);
            if (!waste.exists()) {
                waste.mkdirs();
            }
        }
        return waste;
    }

    @Override
    protected void start() throws PEtALSCDKException {
        getLogger().info("initialize the directory polling.");

        // retrieve the polling period
        String s_polling = getExtensions().getValue(FILETRANSFER_POLLING);
        long period = DEFAULT_POLLING_PERIOD;
        if (!StringHelper.isNullOrEmpty(s_polling)) {
            period = Long.parseLong(s_polling);
            getLogger().finest("set polling period to :" + period + "ms");
        } else {
            getLogger().finest("set polling period to default :" + period + "ms");
        }
        // start the poller
        try {
            poller = new DirectoryPoller(0, period, getWorkingDirectory(), this);
            poller.startProcessing();
            getLogger().info("start polling directory : " + getWorkingDirectory());
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        }

    }

    @Override
    protected void stop() throws PEtALSCDKException {
        poller.stopProcessing();
    }

    /**
     * Send the content of the file, and move it
     * 
     * @param file
     */
    public void processFile(File file) {
        getLogger().info("process file : " + file.getName());

        String transferMode = getExtensions().getValue(FILETRANSFER_TRANSFERMODE);
        getLogger().fine("transfer mode : " + transferMode);
        try {
            if (ENUM_FILETRANSFER_TRANSFERMODE_SOURCE.equalsIgnoreCase(transferMode)) {
                copySource(file);
            } else if (ENUM_FILETRANSFER_TRANSFERMODE_ATTACHMENTS.equalsIgnoreCase(transferMode)) {
                copyAttachment(file);
            } else {
                throw new PEtALSCDKException("Transfer mode not recognized :" + transferMode);
            }
        } catch (Exception e) {
            getLogger().log(Level.SEVERE, e.getMessage(), e);
        }
        moveFile(file);
    }

    /**
     * Send the file as a Source
     * 
     * @param file
     * @throws MessagingException
     * @throws FileNotFoundException
     *             file can not be read
     */
    private void copySource(File file) throws MessagingException, FileNotFoundException {
        // create an exchange set with the ServiceUnit values
        Exchange exchange = createConsumeExchange();
        StreamSource source = new StreamSource(new FileInputStream(file));
        exchange.setInMessageContent(source);
        send(exchange);
    }

    /**
     * Send the file as an attachment
     * 
     * @param file
     * @throws MessagingException
     */
    private void copyAttachment(File file) throws MessagingException {
        // create an exchange set with the ServiceUnit values
        Exchange exchange = createConsumeExchange();
        FileDataSource source = new FileDataSource(file);
        DataHandler attachment = new DataHandler(source);
        exchange.getInMessage().addAttachment(file.getName(), attachment);
        String textString = "<attached-files><file-name>" + file.getName()
                + "</file-name></attached-files>";
        exchange.setInMessageContent(new StreamSource(new ByteArrayInputStream(textString
                .getBytes())));
        send(exchange);
    }

    /**
     * Move the file to the waste directory
     * 
     * @param file
     */
    private void moveFile(File file) {
        String target = "waste-" + addDateToFileName(file.getName());
        file.renameTo(new File(getWasteDirectory(), target));
    }

    /**
     * Add the current time as suffix of the filename
     * 
     * @param filename
     * @return
     */
    private String addDateToFileName(String filename) {
        String result;

        int index = filename.lastIndexOf('.');
        if (index > 0) {
            String prefix = filename.substring(0, filename.lastIndexOf('.'));
            String suffix = filename.substring(filename.lastIndexOf('.'), filename.length());
            result = prefix + System.currentTimeMillis() + suffix;
        } else {
            result = filename + System.currentTimeMillis();
        }

        return result;
    }

}
