/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.listeners;

import static org.ow2.petals.bc.filetransfer.FileTransferConstants.DEFAULT_FILENAME;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_FILETRANSFER_TRANSFERMODE_ATTACHMENTS;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_FILETRANSFER_TRANSFERMODE_SOURCE;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_FILETRANSFER_TRANSFERMODE_BOTH;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_TRANSFERMODE;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_FILENAME;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_GENERATENAME;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.FILETRANSFER_WORKING;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamResult;

import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.bc.filetransfer.util.TransferReport;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 * 
 * @version
 * @since Petals 2.0
 * @author eBMWebsourcing
 * 
 */

public class FileTransferJBIListener extends AbstractJBIListener {

    private Transformer transformer = null;

    /**
     * Process the exchange : copy the received elements
     */
    @Override
    public void onJBIMessage(Exchange exchange) {
        if (exchange.isProviderRole()) {
            this.getLogger().info("Process received exchange: " + exchange.getExchangeId());

            // refuse the others patterns than InOnly
            if (!exchange.isInOnlyPattern()) {
                try {
                    this.getLogger().warning(
                            "Received an unsupported  message exchange pattern: "
                                    + exchange.getExchangePattern() + " from exchange: "
                                    + exchange.getExchangeId());
                } catch (MessagingException e) {
                    this.getLogger().warning(
                            "Received an unsupported message exchange pattern from exchange: "
                                    + exchange.getExchangeId());
                }
                exchange.setError(new Exception("Unsupported exchange pattern"));
            }

            TransferReport transferReport = null;
            try {
                String transferMode = this.getExtensions().getValue(FILETRANSFER_TRANSFERMODE,
                        ENUM_FILETRANSFER_TRANSFERMODE_BOTH);
                if (ENUM_FILETRANSFER_TRANSFERMODE_SOURCE.equalsIgnoreCase(transferMode)) {
                    this.copySource(exchange);
                }
                if (ENUM_FILETRANSFER_TRANSFERMODE_ATTACHMENTS.equalsIgnoreCase(transferMode)) {
                    transferReport = this.copyAttachments(exchange);
                } else if (ENUM_FILETRANSFER_TRANSFERMODE_BOTH.equalsIgnoreCase(transferMode)) {
                    transferReport = this.copyBoth(exchange);
                } else {
                    throw new PEtALSCDKException("Transfer mode not recognized :" + transferMode);
                }
            } catch (Exception e) {
                this.getLogger().log(Level.SEVERE, e.getMessage(), e);
                exchange.setError(new Exception(e.getMessage()));
            }

            if (transferReport != null && transferReport.containsFailedTransfer()) {
                String report = transferReport.generateRawReport();
                this.getLogger().log(Level.SEVERE, report);
                exchange.setError(new Exception(report));
            }

        }
    }

    /**
     * Copy the source as a File
     * 
     * @param exchange
     * @throws MessagingException
     * @throws IOException
     *             problem while writing the file
     * @throws TransformerException
     * @throws ParserConfigurationException
     */
    private void copySource(Exchange exchange) throws MessagingException, IOException,
            TransformerException, ParserConfigurationException {

        File directory = this.getTargetDirectory();
        String fileName = this.getTargetFileName(exchange);

        File target = new File(directory, fileName);
        this.getLogger().info("Source transfer to file : " + target.getAbsolutePath());

        Source source = exchange.getInMessageContentAsSource();
        StreamResult streamResult = new StreamResult(new FileOutputStream(target));
        this.getTransformer().transform(source, streamResult);
        streamResult.getOutputStream().close();
    }

    /**
     * Copy the attachments
     * 
     * @param exchange
     * @throws ParserConfigurationException
     * @throws FileNotFoundException
     * @throws IOException
     */
    private TransferReport copyAttachments(Exchange exchange) throws ParserConfigurationException,
            FileNotFoundException {

        File directory = this.getTargetDirectory();

        TransferReport transferReport = new TransferReport();
        Set<?> names = exchange.getInMessage().getAttachmentNames();
        for (Object name : names) {
            DataHandler attachment = exchange.getInMessage().getAttachment(name.toString());
            this.copyAttachment(attachment, name.toString(), directory, transferReport);
        }

        return transferReport;
    }

    private void copyAttachment(final DataHandler attachment, final String name,
            final File directory, final TransferReport transferReport) {

        String filename = this.addDateToFileName(name.toString());
        File target = new File(directory, filename);
        FileOutputStream os;
        try {
            os = new FileOutputStream(target);
            this.getLogger().info("Attachment transfer to file : " + target.getAbsolutePath());
            attachment.writeTo(os);
            os.close();
            transferReport.addTransferedFile(target, null);
        } catch (IOException e) {
            this.getLogger().warning(e.getMessage());
            transferReport.addTransferedFile(target, e);
        }

    }

    /**
     * Copy the source and the attachments as files
     * 
     * @param exchange
     * 
     * @throws ParserConfigurationException
     * @throws FileNotFoundException
     * @throws MessagingException
     * @throws TransformerFactoryConfigurationError
     */
    private TransferReport copyBoth(Exchange exchange) throws ParserConfigurationException,
            FileNotFoundException, MessagingException, TransformerFactoryConfigurationError {

        TransferReport transferReport = new TransferReport();

        File directory = this.getTargetDirectory();
        String fileName = this.getTargetFileName(exchange);

        // transfer the source
        File target = new File(directory, fileName);
        this.getLogger().info("Source transfer to file : " + target.getAbsolutePath());

        Source source = exchange.getInMessageContentAsSource();
        StreamResult streamResult = new StreamResult(new FileOutputStream(target));
        try {
            this.getTransformer().transform(source, streamResult);
            streamResult.getOutputStream().close();
        } catch (TransformerConfigurationException e) {
            transferReport.addTransferedFile(target, e);
        } catch (TransformerException e) {
            transferReport.addTransferedFile(target, e);
        } catch (IOException e) {
            transferReport.addTransferedFile(target, e);
        }

        // transfer the attachments
        Set<?> names = exchange.getInMessage().getAttachmentNames();
        for (Object name : names) {
            DataHandler attachment = exchange.getInMessage().getAttachment(name.toString());
            this.copyAttachment(attachment, name.toString(), directory, transferReport);
        }

        return transferReport;

    }

    /**
     * Get the target directory.
     * 
     * @param workingParameter
     */
    private File getTargetDirectory() throws FileNotFoundException {
        String dir = this.getExtensions().getValue(FILETRANSFER_WORKING);
        if (StringHelper.isNullOrEmpty(dir)) {
            dir = this.getComponent().getContext().getWorkspaceRoot();
        }

        dir = FileTransferUtils.evaluateFilePath(dir);
        File directory = new File(dir);
        if (!directory.exists()) {
            throw new FileNotFoundException(FILETRANSFER_WORKING + " parameter with value '"
                    + directory + "' is invalid");
        }

        return directory;
    }

    /**
     * Get the target file name for a unique transfer.
     * 
     * @return
     * @throws MessagingException
     */
    private String getTargetFileName(Exchange exchange) throws MessagingException {
        String fileName = this.getExtensions().getValue(FILETRANSFER_FILENAME);
        if (StringHelper.isNullOrEmpty(fileName)) {
            fileName = exchange.getOperationName() + ".xml";
            if (".xml".equals(fileName)) {
                fileName = DEFAULT_FILENAME;
            }
        }

        String generateName = this.getExtensions().getValue(FILETRANSFER_GENERATENAME);
        if (StringHelper.isNullOrEmpty(generateName) || !"false".equals(generateName)) {
            fileName = this.addDateToFileName(fileName);
        }
        return fileName;
    }

    /**
     * Add the current time as suffix of the filename
     * 
     * @param filename
     * @return
     */

    private String addDateToFileName(String filename) {
        String prefix = filename.substring(0, filename.lastIndexOf('.'));
        String suffix = filename.substring(filename.lastIndexOf('.'), filename.length());
        return prefix + System.currentTimeMillis() + suffix;
    }

    private synchronized Transformer getTransformer() throws TransformerFactoryConfigurationError,
            TransformerConfigurationException {
        if (this.transformer == null) {
            this.transformer = TransformerFactory.newInstance().newTransformer();
            Properties props = new Properties();
            props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
            this.transformer.setOutputProperties(props);
        }
        return this.transformer;
    }

}
